//Blink LED in the Start Task

#include  <bsp_os.h>
#include  "bsp.h"

#include  <cpu/include/cpu.h>
#include  <common/include/common.h>
#include  <kernel/include/os.h>
#include  <kernel/include/os_trace.h>

#include  <common/include/lib_def.h>
#include  <common/include/rtos_utils.h>
#include  <common/include/toolchains.h>
#include  "retargetserial.h"

/********************************************************************************************************
*                                             LOCAL DEFINES
********************************************************************************************************/
#define  START_TASK_PRIO              21u								// Start Task priority
#define  START_TASK_STK_SIZE         256u								// Start Task stack size
#define  MY_TASK_STK_SIZE			 256u

/********************************************************************************************************
*                                        LOCAL GLOBAL VARIABLES
********************************************************************************************************/
static  CPU_STK  StartTaskStk[START_TASK_STK_SIZE];  					// Start Task Stack.
static  CPU_STK	LampTaskStk[MY_TASK_STK_SIZE];

static  OS_TCB  StartTaskTCB;											// Start Task TCB.
static  OS_TCB	LampTaskTCB;

static 	OS_SEM  Button_sem;

/*********************************************************************************************************
*                                       LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************/
static void StartTask(void *p_arg);										// Start Task prototype
static void	LampTask (void *p);

int main (void)
{
    RTOS_ERR  err;

    BSP_SystemInit();                                           		// Initialize System.
    CPU_Init();                                                 		// Initialize CPU.

    RETARGET_SerialInit();
    RETARGET_SerialCrLf(1);

    OS_TRACE_INIT();
    OSInit(&err);                                               		// Initialize the Kernel.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    OSTaskCreate(&StartTaskTCB,                          				// Create the Start Task.
                 "Start Task",
                  StartTask,
                  DEF_NULL,
                  START_TASK_PRIO,
                 &StartTaskStk[0],
                 (START_TASK_STK_SIZE / 10u),
                  START_TASK_STK_SIZE,
                  0u,
                  0u,
                  DEF_NULL,
                 (OS_OPT_TASK_STK_CLR),
                 &err);
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    OSStart(&err);                                              		// Start the kernel.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    return (1);
}

static void StartTask (void *p_arg)
{
    RTOS_ERR  err;
    PP_UNUSED_PARAM(p_arg);                                     		// Prevent compiler warning.
    BSP_TickInit();                                             		// Initialize Kernel tick source.

#if (OS_CFG_STAT_TASK_EN == DEF_ENABLED)
    OSStatTaskCPUUsageInit(&err);                               		// Initialize CPU Usage.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), ;);	//   Check error code.
#endif

#ifdef CPU_CFG_INT_DIS_MEAS_EN
    CPU_IntDisMeasMaxCurReset();                                		// Initialize interrupts disabled measurement.
#endif

    Common_Init(&err);                                          		// Call common module initialization example.
    APP_RTOS_ASSERT_CRITICAL(err.Code == RTOS_ERR_NONE, ;);

    BSP_OS_Init();                                              		// Initialize the BSP. It is expected that the BSP ...
                                                                		// ... will register all the hardware controller to ...
                                                                		// ... the platform manager at this moment.

    OSSemCreate (&Button_sem, "My sem", 0, &err);						// init semaphore object
    GPIO_PinModeSet(BSP_GPIO_PB0_PORT, BSP_GPIO_PB0_PIN, gpioModeInput, 0);
    GPIO_ExtIntConfig(BSP_GPIO_PB0_PORT, BSP_GPIO_PB0_PIN, 14, false, true, true); // set falling edge interrupt
    NVIC_EnableIRQ(GPIO_EVEN_IRQn);										// enable BUTTON0 interrupt in NVIC

    OSTaskCreate(&LampTaskTCB,              							// Create custom task
                 "Lamp Task",
                  LampTask,
                  0u,
                  START_TASK_PRIO+2,
                  &LampTaskStk[0u],
                  LampTaskStk[MY_TASK_STK_SIZE / 10u],
                  MY_TASK_STK_SIZE,
                  0u,
                  0u,
                  0u,
                  (OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                  &err);

    while (1)
    {
        OSTimeDlyHMSM(1u, 0u, 0u, 100u, OS_OPT_TIME_HMSM_STRICT, &err);
        APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), ;); //   Check error code.
    }
}

static void LampTask (void *p_arg)
{
    RTOS_ERR err;
    PP_UNUSED_PARAM(p_arg);                                     		// Prevent compiler warning

    while (DEF_TRUE)
    {
    	OSSemPend(&Button_sem, 0, OS_OPT_PEND_BLOCKING, 0, &err);		// wait for semaphore
    	BSP_LedToggle(1);
    }
}

void GPIO_EVEN_IRQHandler(void)
{
	RTOS_ERR err;
	CPU_SR_ALLOC();                                             		// Allocate storage for CPU status register
	CPU_CRITICAL_ENTER();
	OSIntEnter();
	CPU_CRITICAL_EXIT();
	GPIO_IntClear(1<<BSP_GPIO_PB0_PIN);									// clear IF
	OSSemPost(&Button_sem, OS_OPT_POST_ALL, &err);  					// unlock the LampTask
	APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), ;); 	//   Check error code.
	OSIntExit();
}
